from .action import *
from .application import *
from .button import *
from .checkbox import *
from .combobox import *
from .dialog import *
from .frame import *
from .groupbox import *
from .icon import *
from .label import *
from .layout import *
from .list import *
from .mainwindow import *
from .menu import *
from .messagebox import *
from .number import *
from .object import *
from .pixmap import *
from .progressbar import *
from .scrollarea import *
from .settings import *
from .splitter import *
from .table import *
from .tabs import *
from .text import *
from .textarea import *
from .tree import *
from .widget import *
from .worker import *

__version__ = '1.0.0'



from .qt import QtCore
from .qt import QtWidgets
from .qt import bind

from .widget import BaseWidget

__all__ = ['Text']

@bind(QtWidgets.QLineEdit)
class Text(BaseWidget):

    def __init__(self, value=None, *, readonly=False, clearable=False,
                 changed=None, editing_finished=None, **kwargs):
        super().__init__(**kwargs)
        self.readonly = readonly
        self.clearable = clearable
        if value is not None:
            self.value = value
        self.changed = changed
        self.editing_finished = editing_finished
        # Connect signals
        self.qt.textChanged.connect(self.__handle_changed)
        self.qt.editingFinished.connect(self.__handle_editing_finished)

    @property
    def value(self):
        return self.qt.text()

    @value.setter
    def value(self, value):
        self.qt.setText(value)

    @property
    def readonly(self):
        return self.qt.isReadOnly()

    @value.setter
    def readonly(self, value):
        self.qt.setReadOnly(value)

    @property
    def clearable(self):
        return self.qt.clearButtonEnabled()

    @clearable.setter
    def clearable(self, value):
        self.qt.setClearButtonEnabled(value)

    @property
    def changed(self):
        return self.__changed

    @changed.setter
    def changed(self, value):
        self.__changed = value

    def __handle_changed(self, text):
        if callable(self.changed):
            self.changed(text)

    @property
    def editing_finished(self):
        return self.__editing_finished

    @editing_finished.setter
    def editing_finished(self, value):
        self.__editing_finished = value

    def __handle_editing_finished(self):
        if callable(self.editing_finished):
            self.editing_finished()

    def clear(self):
        self.qt.clear()



import threading

from .qt import QtCore
from .qt import bind

from .object import Object

__all__ = ['Worker']

@bind(QtCore.QObject)
class Worker(Object):

    def __init__(self, *, target=None, finished=None, failed=None, **kwargs):
        super().__init__(**kwargs)
        self.target = target
        self.finished = finished
        self.failed = failed
        self.__lock = threading.RLock()
        self.__thread = None
        self.__stop_requested = False

    @property
    def target(self):
        return self.__target

    @target.setter
    def target(self, value):
        self.__target = value

    @property
    def finished(self):
        return self.__finished

    @finished.setter
    def finished(self, value):
        self.__finished = value

    @property
    def failed(self):
        return self.__failed

    @failed.setter
    def failed(self, value):
        self.__failed = value

    def start(self):
        with self.__lock:
            if not self.__thread:
                self.__thread = threading.Thread(target=self.__run)
            self.__stop_requested = False
            self.__thread.start()

    def stop(self):
        self.__stop_requested = True

    def join(self):
        try:
            self.__thread.join()
        except AttributeError:
            pass

    @property
    def stopping(self):
        return self.__stop_requested

    @property
    def alive(self):
        with self.__lock:
            if self.__thread:
                return self.__thread.alive()
        return False

    def __run(self):
        try:
            if self.target is None:
                self.run()
            else:
                self.target(self)
        except Exception as e:
            self.emit('failed', e)
        finally:
            with self.__lock:
                self.__stop_requested = False
                self.__thread = None
                self.emit('finished')

    def run(self):
        pass



from .qt import QtCore
from .qt import QtWidgets
from .qt import bind

from .widget import BaseWidget, Widget

__all__ = [
    'Tab',
    'Tabs'
]

class Tab(Widget):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def title(self):
        return self.qt.windowTitle()

    @title.setter
    def title(self, value):
        self.qt.setWindowTitle(value)
        # Update tab title
        parent = self.qt.parent()
        if parent:
            if parent.parent():
                index = parent.parent().indexOf(self.qt)
                parent.parent().setTabText(index, value)

@bind(QtWidgets.QTabWidget)
class Tabs(BaseWidget):

    def __init__(self, *items, changed=None, **kwargs):
        super().__init__(**kwargs)
        for items in items:
            self.append(items)
        self.changed = changed
        # Connect signals
        self.qt.currentChanged.connect(self.__handle_changed)

    @property
    def items(self):
        return list(self)

    def append(self, tab):
        assert isinstance(tab, Tab), "must be of type Tab"
        self.qt.addTab(tab.qt, tab.title)

    def insert(self, index, tab):
        assert isinstance(tab, Tab), "must be of type Tab"
        if index < 0:
            index = max(0, len(self) + index)
        self.qt.insertTab(index, tab.qt, tab.title)

    def remove(self, tab):
        assert isinstance(tab, Tab), "must be of type Tab"
        if tab is not None:
            index = self.qt.indexOf(tab.qt)
            self.qt.removeTab(index)

    @property
    def current(self):
        index = self.qt.currentIndex()
        return self[index]

    @current.setter
    def current(self, item):
        index = self.index(item)
        self.qt.setCurrentIndex(index)

    def index(self, item):
        index = self.qt.indexOf(item.qt)
        if index < 0:
            raise ValueError("item not in tabs")
        return index

    @property
    def changed(self):
        return self.__changed

    @changed.setter
    def changed(self, changed):
        self.__changed = changed

    def __handle_changed(self, index):
        if callable(self.changed):
            self.changed(index)

    def __getitem__(self, key):
        widget = self.qt.widget(key)
        if not widget:
            raise KeyError(key)
        return widget.property(self.QtPropertyKey)

    def __setitem__(self, key, value):
        del self[index]
        self.insert(key, value)

    def __delitem__(self, key):
        self.qt.removeTab(key)

    def __len__(self):
        return self.qt.count()

    def __iter__(self):
        return (self[index] for index in range(len(self)))



import os

from .qt import QtWidgets
from .qt import bind

from .widget import BaseWidget

__all__ = ['TextArea']

@bind(QtWidgets.QTextEdit)
class TextArea(BaseWidget):

    def __init__(self, value=None, *, readonly=False, richtext=False, **kwargs):
        super().__init__(**kwargs)
        self.readonly = readonly
        self.richtext = richtext
        if value is not None:
            self.value = value

    @property
    def value(self):
        return self.qt.toPlainText()

    @value.setter
    def value(self, value):
        self.qt.setPlainText(value)

    @property
    def readoly(self):
        return self.qt.readOnly()

    @readoly.setter
    def readoly(self, value):
        self.qt.setReadOnly(value)

    @property
    def richtext(self):
        return self.qt.acceptRichText()

    @richtext.setter
    def richtext(self, value):
        self.qt.setAcceptRichText(value)

    def append(self, text):
        self.text = os.linesep.join((self.text, text))

    def insert(self, text):
        self.qt.insertPlainText(text)

    def clear(self):
        self.qt.clear()

    def select_all(self):
        self.qt.selectAll()

    def redo(self):
        self.qt.redo()

    def undo(self):
        self.qt.undo()



from .qt import QtGui
from .qt import QtWidgets
from .qt import bind

from .widget import Widget

__all__ = ['ScrollArea']

@bind(QtWidgets.QScrollArea)
class ScrollArea(Widget):

    def __init__(self, layout=None, **kwargs):
        super().__init__(**kwargs)
        self.qt.setWidgetResizable(True)
        self.qt.setWidget(Widget().qt)
        self.qt.setBackgroundRole(QtGui.QPalette.Base) # fix background
        if layout is not None:
            self.layout = layout

    @property
    def layout(self):
        return self.qt.widget().property(self.QtPropertyKey).layout

    @layout.setter
    def layout(self, value):
        if self.qt.widget():
            self.qt.widget().property(self.QtPropertyKey).layout = value



from .qt import QtWidgets
from .qt import bind

from .frame import Frame

__all__ = ['Label']

@bind(QtWidgets.QLabel)
class Label(Frame):

    def __init__(self, text=None, *, word_wrap=None, **kwargs):
        super().__init__(**kwargs)
        if text is not None:
            self.text = text
        if word_wrap is not None:
            self.word_wrap = word_wrap

    @property
    def text(self):
        return self.qt.text()

    @text.setter
    def text(self, value):
        self.qt.setText(value)

    @property
    def word_wrap(self):
        return self.qt.wordWrap()

    @word_wrap.setter
    def word_wrap(self, value):
        self.qt.setWordWrap(value)

    @property
    def selected_text(self):
        return self.qt.selectedText()
